import re
import time
from typing import Final, Dict
import pandas as pd
from bs4 import BeautifulSoup, ResultSet
from selenium.webdriver.chrome.webdriver import WebDriver
from house_info.base import BaseSelRequest
from house_info.utils import PageList, LIST_IN_TUPLE, DataFrame


class MyHomeRequest(BaseSelRequest):
    """
    마이홈 포털 정보 수집
    :ChromeDriver 필요
    """

    URL: Final = "https://www.myhome.go.kr/hws/portal/sch/selectRsdtRcritNtcView.do"

    def __init__(
        self,
        chrome_driver_path: str,
        page: int = 1,
        types: str = "student",
        region: str = "seoul",
    ) -> None:
        super(MyHomeRequest, self).__init__(chrome_driver_path, page)
        self.types = types
        self.region = region

        assert self.types in ["student", "couple"], "types는 student 또는 couple 이어야함"
        assert self.region in ["seoul", "kkd"], "region는 seoul 또는 kkd 이어야함"

    def get_post_list(self, soup: BeautifulSoup) -> ResultSet:
        post_list = soup.find("tbody", attrs={"id": "schTbody"}).find_all("tr")
        if not post_list:
            return None
        return post_list

    def move_next_page(self, browser: WebDriver, page: int) -> WebDriver:
        browser.execute_script(f"fnSearch({page})")
        return browser

    def click_category(self, browser: WebDriver) -> WebDriver:
        if self.region == "seoul" and self.types == "student":
            browser.execute_script("fnSchKoreaMapClick('11');")  # 서울
            time.sleep(1.5)
            browser.execute_script("setUserTy('FIXES100001');")  # 대학생
        elif self.region == "seoul" and self.types == "couple":
            browser.execute_script("fnSchKoreaMapClick('11');")
            time.sleep(1.5)
            browser.execute_script("setUserTy('FIXES100002');")  # 신혼부부
        elif self.region == "kkd" and self.types == "student":
            browser.execute_script("fnSchKoreaMapClick('41');")  # 경기
            time.sleep(1.5)
            browser.execute_script("setUserTy('FIXES100001');")
        elif self.region == "kkd" and self.types == "couple":
            browser.execute_script("fnSchKoreaMapClick('41');")
            time.sleep(1.5)
            browser.execute_script("setUserTy('FIXES100002');")
        return browser

    def create_post_list_sources(self) -> PageList:
        browser = self.browser_open_headless()
        browser = self.click_category(browser)
        time.sleep(0.5)
        page_source = self.get_page_source(browser)
        parsed_html = self.parse_html(page_source)
        post_list = self.get_post_list(parsed_html)
        post_list_sources = PageList([post_list])

        for page in range(2, self.page + 1):
            self.move_next_page(browser, page)
            time.sleep(1)
            # TODO: neeed to extract func
            page_source = self.get_page_source(browser)
            parsed_html = self.parse_html(page_source)
            post_list = self.get_post_list(parsed_html)

            if not post_list:
                break
            post_list_sources.append(post_list)
        return post_list_sources

    def __call__(self) -> PageList:
        return self.create_post_list_sources()


class MyHomeTable:
    def __init__(self, page_sources: PageList):
        self.page_sources = page_sources

    def get_data(self) -> LIST_IN_TUPLE:
        (
            _type,
            status,
            region,
            title,
            link,
            _,
            registration_date,
            release_date,
            supplier,
        ) = ([], [], [], [], [], [], [], [], [])
        p = re.compile("[0-9]+")

        for page_source in self.page_sources:
            for post in page_source:
                items = post.find_all("td")
                _type.append(items.pop(0).get_text().strip())
                status.append(items.pop(0).get_text().strip())
                region.append(items.pop(0).get_text().strip())

                title_tag = items.pop(0)
                detail_id = p.findall(title_tag.find("a").attrs["href"])[0]
                url = f"https://www.myhome.go.kr/hws/portal/sch/selectRsdtRcritNtcDetailView.do?pblancId={detail_id}"
                link.append(url)
                title.append(title_tag.get_text().strip())

                _.append(items.pop(0))

                registration_date.append(items.pop(0).get_text().strip())
                release_date.append(items.pop(0).get_text().strip())
                supplier.append(items.pop(0).get_text().strip())
        return (
            _type,
            status,
            region,
            title,
            registration_date,
            release_date,
            supplier,
            link,
        )

    def get_data_dict(self) -> Dict:
        data = self.get_data()
        data_dict = {
            "공급유형": data[0],
            "진행상태": data[1],
            "지역": data[2],
            "공고명": data[3],
            "모집공고일자": data[4],
            "당첨발표일자": data[5],
            "공급기관": data[6],
            "링크": data[7],
        }
        return data_dict

    def create_data_frame(self) -> DataFrame:
        data_dict = self.get_data_dict()
        data_frame = pd.DataFrame(data=data_dict)
        return data_frame


class MyHomeDataManager:
    def __init__(
        self,
        chrome_driver_path: str,
        page: int = 1,
        types: str = "student",
        region: str = "seoul",
    ) -> None:
        self.request = MyHomeRequest(chrome_driver_path, page, types, region)

    def create_page_sources(self) -> PageList:
        page_sources = self.request()
        return page_sources

    def get_data_frame(self, page_sources: PageList) -> DataFrame:
        data_table = MyHomeTable(page_sources)
        data_table = data_table.create_data_frame()
        return data_table

    def export_csv(self, path: str) -> None:
        page_sources = self.create_page_sources()
        data_table = self.get_data_frame(page_sources)

        data_table.to_csv(path, index=False)



import logging
from typing import Tuple, List
import pandas as pd


logger = logging.getLogger(__name__)
DataFrame = pd.DataFrame
SHColumn = Tuple[int, str, str, str]
LIST_IN_TUPLE = Tuple[List, List, List, List, List, List]


class PageList(list):
    def __str__(self) -> str:
        return f"1페이지부터 {len(self)}페이지까지 저장됨"

    def __repr__(self) -> str:
        return f"1페이지부터 {len(self)}페이지까지 저장됨"



import time
from typing import Final, List, Tuple, Dict
from bs4 import ResultSet
import pandas as pd
from selenium.webdriver.chrome.webdriver import WebDriver
from house_info.base import BaseSelRequest
from house_info.utils import DataFrame, LIST_IN_TUPLE, PageList


class StationAreaRequest(BaseSelRequest):
    """
    역세권 청년주택 정보 수집
    민간, 공공 포함
    :ChromeDriver 필요
    """

    URL: Final = "https://soco.seoul.go.kr/youth/bbs/BMSR00015/list.do?menuNo=400008"

    def __init__(self, chrome_driver_path: str, page: int = 1) -> None:
        super(StationAreaRequest, self).__init__(chrome_driver_path, page)

    def get_post_list(self, soup) -> List:
        post_list = soup.find("tbody", attrs={"id": "boardList"}).find_all("tr")
        if not post_list:
            return None
        return post_list

    def move_next_page(self, browser: WebDriver, page: int):
        browser.execute_script(f"cohomeList({page})")
        return browser

    def create_post_list_sources(self) -> PageList:
        browser = self.browser_open_headless()
        page_source = self.get_page_source(browser)
        parsed_html = self.parse_html(page_source)
        post_list = self.get_post_list(parsed_html)
        post_list_sources = PageList([post_list])

        for page in range(2, self.page + 1):
            self.move_next_page(browser, page)
            time.sleep(0.1)
            # TODO: neeed to extract func
            page_source = self.get_page_source(browser)
            parsed_html = self.parse_html(page_source)
            post_list = self.get_post_list(parsed_html)

            if not post_list:
                break
            post_list_sources.append(post_list)
        return post_list_sources

    def __call__(self):
        return self.create_post_list_sources()


class StationAreaTable:
    def __init__(self, page_sources: LIST_IN_TUPLE) -> None:
        self.page_sources = page_sources

    def get_data(self) -> Tuple[List]:
        index, _type, title, link, registration_date, subscription_date, manager = (
            [],
            [],
            [],
            [],
            [],
            [],
            [],
        )

        for page_source in self.page_sources:
            for post in page_source:
                items = post.find_all("td")
                index.append(items.pop(0).get_text().strip())
                _type.append(items.pop(0).get_text().strip())

                title_tag = items.pop(0)
                url = (
                    "https://soco.seoul.go.kr/youth/bbs/BMSR00015/"
                    + title_tag.find("a").attrs["href"]
                )
                link.append(url)

                title.append(title_tag.get_text().strip())
                registration_date.append(items.pop(0).get_text().strip())
                subscription_date.append(items.pop(0).get_text().strip())
                manager.append(items.pop(0).get_text().strip())
        return index, _type, title, registration_date, subscription_date, manager, link

    def get_data_dict(self) -> Dict[str, List[str]]:
        data = self.get_data()
        data_dict = {
            "인덱스": data[0],
            "유형": data[1],
            "제목": data[2],
            "게시일": data[3],
            "청약신청일": data[4],
            "담당자": data[5],
            "링크": data[6],
        }
        return data_dict

    def create_data_frame(self) -> DataFrame:
        data_dict = self.get_data_dict()
        data_frame = pd.DataFrame(data=data_dict)
        return data_frame


class StationAreaDataManager:
    def __init__(self, chrome_driver_path: str, page: int = 1) -> None:
        self.request = StationAreaRequest(chrome_driver_path, page)

    def create_page_sources(self) -> ResultSet:
        page_sources = self.request()
        return page_sources

    def get_data_frame(self, page_sources) -> DataFrame:
        data_table = StationAreaTable(page_sources)
        data_table = data_table.create_data_frame()
        return data_table

    def export_csv(self, path: str) -> None:
        page_sources = self.create_page_sources()
        data_table = self.get_data_frame(page_sources)

        data_table.to_csv(path, index=False)



from typing import Tuple, List
from requests import Session
from concurrent.futures import ThreadPoolExecutor
from house_info.utils import PageList


class MultiThreadingRequestMixin:
    max_workers = 8
    executor = ThreadPoolExecutor(max_workers=8)

    def fetcher(self, params: Tuple[Session, str]):
        session = params[0]
        url = params[1]

        with session.get(url) as response:
            return response.text

    def get_pages_with_multi_threading(self, urls: List[str]) -> PageList:
        executor = ThreadPoolExecutor(max_workers=8)

        with Session() as session:
            params = [(session, url) for url in urls]
            pages = PageList(executor.map(self.fetcher, params))

        return pages

    @classmethod
    def change_max_workers(cls, num: int):
        cls.executor = ThreadPoolExecutor(max_workers=num)



from typing import List, Final
from collections import defaultdict
import pandas as pd
from requests import Session, Response
from bs4 import BeautifulSoup, ResultSet
from house_info.utils import PageList, logger, DataFrame, SHColumn
from house_info.mixins import MultiThreadingRequestMixin


class SHRequest(MultiThreadingRequestMixin):
    URL: Final = "https://www.i-sh.co.kr/main/lay2/program/S1T294C297/www/brd/m_247/list.do?page="

    def __init__(self, page: int = 1, thread: bool = False) -> None:
        self.page: int = page
        self.__pages: List[int] = self.__get_page_numbers()
        self.thread: bool = thread

    def __get_page_numbers(self) -> List[int]:
        """
        page number 리스트 뽑기
        """
        page_numbers = [i for i in range(1, self.page + 1)]
        return page_numbers

    def __get_pages(self) -> PageList:
        urls = self.get_urls()
        pages = PageList([self.send_requests(url).text for url in urls])
        return pages

    def get_urls(self) -> List[str]:
        """
        page number list를 기준으로 page url list up 하기
        """
        urls = [SHRequest.URL + str(page) for page in self.__pages]
        return urls

    def send_requests(self, url: str) -> Response:
        """
        해당 url로 요청을 보내서 html 소스 가져오기
        """
        with Session() as s:
            response = s.get(url)
            return response

    def __call__(self) -> PageList:
        if self.thread:
            urls = self.get_urls()
            return self.get_pages_with_multi_threading(urls)
        return self.__get_pages()


class SHPagePreprocessorMixin:
    """
    Page Source 전처리 Mixin
    """

    def get_title(self, raw_html: ResultSet = None, index: int = 1) -> str:
        """
        '사이트 스타일이 바뀌면 코드를 다시 작성해야함'

        html에서 title 뽑아오기
        index[0] -> table column
        index[1] -> title
        """
        try:
            title = raw_html[index].find_all("a", attrs={"class": "ellipsis icon"})
            if title == []:
                title = raw_html[index].find_all("a", attrs={"class": "ellipsis"})
            title = title[0].get_text()
            title = title.rstrip().strip().split("\r")
            title = list(map(lambda x: x.strip().rstrip(), title))
            title = list(filter(lambda x: x != "", title))

            if len(title) == 2:
                _, title = title
            else:
                _, title = False, title.pop()
            return title

        except ValueError as e:
            logger.debug("글제목 없음")
            return title

    def get_index(self, raw_html: ResultSet = None, index: int = 1) -> int:
        """
        게시물 index 가져오기
        """
        index = raw_html[index].find("td")
        index = index.get_text()
        try:
            if index.isnumeric():
                return int(index)
        except ValueError as e:
            logger.debug("index가 number가 아님")
            return 9999999

    def get_department(self, raw_html: ResultSet = None, index: int = 1) -> str:
        """
        담당부서 가져오기
        """
        try:
            department = raw_html[index].find_all("td")[2].get_text()
            department = department.strip().rstrip()
            return department
        except ValueError as e:
            logger.debug("담당부서 없음")
            return "담당부서 없음"

    def get_registration_date(self, raw_html: ResultSet = None, index: int = 1) -> str:
        """
        등록일 가져오기
        """
        try:
            registration_date = (
                raw_html[index].find_all("td", attrs={"class": "num"})[0].get_text()
            )
            registration_date = registration_date.strip().rstrip()
            return registration_date
        except ValueError as e:
            logger.debug("등록 날짜 없음")
            return "2222-12-12"


class SHTable(SHPagePreprocessorMixin):
    """
    Pandas DataFrame 구조로 Table 객체 생성
    page_sources : SHRequest로 받아온 페이지 목록 리스트
    """

    def __init__(self, page_sources: PageList):
        assert page_sources.__class__ == PageList, "SHRequest로 받아온 Page Source만 전처리 가능"
        self.page_sources = [
            BeautifulSoup(page, "html.parser").find_all("tr") for page in page_sources
        ]

    def create_data_frame(self) -> DataFrame:
        """
        Dataframe 생성
        """
        data = self._get_data_dict(self.page_sources)
        data_frame = pd.DataFrame(data=data)
        return data_frame

    def _get_data_dict(self, pages) -> defaultdict:
        """
        page별로 전처리하여 data_dict 생성
        DataFrame의 column: 인덱스, 제목, 담당부서, 등록일자
        """
        data_dict = defaultdict(list)

        for page in pages:
            result = self._get_row_data(page)
            data_dict["인덱스"].extend(result[0])
            data_dict["제목"].extend(result[1])
            data_dict["담당부서"].extend(result[2])
            data_dict["등록일자"].extend(result[3])
        return data_dict

    def _get_row_data(self, page) -> SHColumn:
        """
        page에서 필요한 요소 추출
        """
        # post_length: page내 게시물 개수 -> 10개의 게시물이어야함
        # index 1 ~ 10
        post_length = 10

        index_list, title_list, department_list, registration_date_list = [], [], [], []
        for i in range(1, post_length + 1):
            index_list.append(self.get_index(page, i))
            title_list.append(self.get_title(page, i))
            department_list.append(self.get_department(page, i))
            registration_date_list.append(self.get_registration_date(page, i))
        result = (index_list, title_list, department_list, registration_date_list)
        return result


class SHDataManager:
    def __init__(self, page: int = 1, thread: bool = False) -> None:
        self.request = SHRequest(page, thread)

    def create_page_sources(self) -> ResultSet:
        page_sources = self.request()
        return page_sources

    def get_data_frame(self, page_sources) -> DataFrame:
        data_table = SHTable(page_sources)
        data_table = data_table.create_data_frame()
        return data_table

    def export_csv(self, path: str) -> None:
        page_sources = self.create_page_sources()
        data_table = self.get_data_frame(page_sources)

        data_table.to_csv(path, index=False)



from setuptools import setup, find_packages

setup(
    name="house-info",
    version="0.0.3",
    description="공공 주택 공고 정보 수집",
    author="Jungminchae",
    author_email="minggae0629@gmail.com",
    url="https://github.com/c2-house/house-info",
    install_requires=["selenium", "requests", "beautifulsoup4", "pandas"],
    include_package_data=True,
    packages=find_packages(),
    keywords=["house-info"],
    python_requires=">=3",
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from typing import List
from bs4 import BeautifulSoup, ResultSet
from selenium.webdriver import Chrome
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.chrome.webdriver import WebDriver


class BaseSelRequest:
    """
    selenium을 사용하는 요청 Base
    """

    URL: str = "url"

    def __init__(self, chrome_driver_path: str, page: int = 1) -> None:
        self.__service = Service(chrome_driver_path)
        self.page = page

    def browser_open_headless(self) -> WebDriver:
        chrome_option = Options()
        chrome_option.add_argument("--headless")
        browser = Chrome(service=self.__service, options=chrome_option)
        browser.implicitly_wait(2)
        browser.get(self.URL)
        return browser

    def get_page_source(self, browser: WebDriver):
        page_source = browser.page_source
        return page_source

    def parse_html(self, page_source) -> ResultSet:
        soup = BeautifulSoup(page_source, "html.parser")
        return soup

    def get_post_list(self, soup):
        pass

    def move_next_page(self, browser, page):
        browser.execute_script(f"javascript({page})")
        return browser

    def create_post_list_sources(self) -> List[ResultSet]:
        pass
